#include "PX_Object_Battle.h"

px_void PX_Object_BattleListOnChanged(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pBattleObject = (PX_Object*)ptr;
	px_int index = PX_Object_ListGetCurrentSelectIndex(pObject);
	PX_Object_Battle* pdesc = PX_ObjectGetDesc(PX_Object_Battle, pBattleObject);
	px_int count = PX_GameInstanceDataGetCount();
	PX_GameInstanceData* plocaldata, * popponentdata;

	plocaldata = PX_GameInstanceDataGetLocalPtr();
	popponentdata = PX_GameInstanceDataGetPtr();


	if (plocaldata->binsize==0)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "battle.select major"), 0, 0);
		return;
	}

	if (index>=0&&index<count)
	{
		px_int i;
		PX_GAME_RESULT result = PX_GameRunTest(&pdesc->pruntime->mp_game, plocaldata, popponentdata + index);
		pdesc->result[index] = result;
		PX_GameStart(&pdesc->game, plocaldata, popponentdata+index);

		plocaldata->match = 0;
		plocaldata->win = 0;
		for (i=0;i<PX_COUNTOF(pdesc->result);i++)
		{
			switch (pdesc->result[i])
			{
			case PX_GAME_RESULT_BLUEWIN:
			case PX_GAME_RESULT_TIE:
				plocaldata->match++;
				break;
			case PX_GAME_RESULT_REDWIN:
				plocaldata->match++;
				plocaldata->win++;
				break;
			default:
				break;
			}
		}
	}
	else
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "battle.select opponent"), 0, 0);
	}


}



px_void PX_Object_BattleListItemOnRender(px_surface* psurface, PX_Object* pObject, px_dword elapsed)
{
	PX_Object* pBattleObject = pObject->User_ptr;
	PX_GameInstanceData* pdata = PX_Object_ListItemGetData(pObject);
	px_int index = PX_Object_ListItemGetIndex(pObject);
	PX_Object_Battle* pdesc = PX_ObjectGetDesc(PX_Object_Battle, pBattleObject);
	px_texture staticTex = { 0 };
	px_char content[256] = { 0 };

	staticTex.height = 48;
	staticTex.limit_bottom = 47;
	staticTex.limit_right = 47;
	staticTex.MP = 0;
	staticTex.width = 48;
	staticTex.surfaceBuffer = (px_color*)pdata->profile_data;

	PX_TextureRender(psurface, &staticTex, (px_int)(pObject->x + 8  + staticTex.width / 2 + 10), (px_int)(pObject->y + pObject->Height / 2), PX_ALIGN_CENTER, 0);

	switch (pdesc->result[index])
	{
	case PX_GAME_RESULT_UNKNOW:
	{
		PX_sprintf2(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.info"), \
			PX_STRINGFORMAT_STRING(pdata->name),
			PX_STRINGFORMAT_STRING(pdata->author)
		);

		PX_FontModuleDrawText(psurface, pdesc->fm, (px_int)(pObject->x + 8 + staticTex.width + 20), (px_int)(pObject->y + pObject->Height / 2), PX_ALIGN_LEFTMID, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	}
	break;
	case PX_GAME_RESULT_TIE:
	{
		PX_sprintf3(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.info_result"), \
			PX_STRINGFORMAT_STRING(pdata->name),
			PX_STRINGFORMAT_STRING(pdata->author),
			PX_STRINGFORMAT_STRING(PX_JsonGetString(pdesc->language, "battle.tie"))
		);
		PX_FontModuleDrawText(psurface, pdesc->fm, (px_int)(pObject->x + 8 + staticTex.width + 20), (px_int)(pObject->y + pObject->Height / 2), PX_ALIGN_LEFTMID, content, PX_COLOR(255,255,255,0));
	}
	break;
	case PX_GAME_RESULT_REDWIN:
	{
		PX_sprintf3(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.info_result"), \
			PX_STRINGFORMAT_STRING(pdata->name),
			PX_STRINGFORMAT_STRING(pdata->author),
			PX_STRINGFORMAT_STRING(PX_JsonGetString(pdesc->language, "battle.win"))
		);
		PX_FontModuleDrawText(psurface, pdesc->fm, (px_int)(pObject->x + 8 + staticTex.width + 20), (px_int)(pObject->y + pObject->Height / 2), PX_ALIGN_LEFTMID, content, PX_COLOR(255, 0, 255, 0));
	}
	break;
	case PX_GAME_RESULT_BLUEWIN:
	{
		PX_sprintf3(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.info_result"), \
			PX_STRINGFORMAT_STRING(pdata->name),
			PX_STRINGFORMAT_STRING(pdata->author),
			PX_STRINGFORMAT_STRING(PX_JsonGetString(pdesc->language, "battle.lose"))
		);
		PX_FontModuleDrawText(psurface, pdesc->fm, (px_int)(pObject->x + 8 + staticTex.width + 20), (px_int)(pObject->y + pObject->Height / 2), PX_ALIGN_LEFTMID, content, PX_COLOR(255, 255, 0, 0));
	}
	break;
	default:
		break;
	}


	
}

px_bool PX_Object_BattleListItemOnCreate(px_memorypool* mp, PX_Object* ItemObject, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	ItemObject->Func_ObjectRender = PX_Object_BattleListItemOnRender;
	ItemObject->User_ptr = userptr;
	return PX_TRUE;
}


px_void PX_Object_BattleUpdate( PX_Object* pObject, px_dword elapsed)
{
	PX_Object_Battle *pdesc = PX_ObjectGetDesc(PX_Object_Battle, pObject);
	if (elapsed>1000)
	{
		return;
	}
	if (pdesc->game.state==PX_GAME_STATE_RUN)
	{
		PX_GameUpdate(&pdesc->game, (px_dword)(elapsed* pdesc->gamespeed));
		pdesc->auto_observer_elapsed = 0;
	}

	if (pdesc->game.state==PX_GAME_STATE_END)
	{
		if (pdesc->auto_observer_mode)
		{
			pdesc->auto_observer_elapsed += elapsed;
			if (pdesc->auto_observer_elapsed>=3600)
			{
				pdesc->auto_observer_elapsed = 0;
				px_int count = PX_GameInstanceDataGetCount();
				PX_GameInstanceData* pdata = PX_GameInstanceDataGetPtr();
				if (count>1)
				{
					px_int p1 = (PX_rand()/10) % count;
					px_int p2 = (PX_rand()/10) % count;
					if (p1==p2)
					{
						px_int p2 = (PX_rand() / 10) % count;
					}
					if (p1 == p2)
					{
						px_int p2 = (PX_rand() / 10) % count;
					}
					PX_GameStart(&pdesc->game, pdata+p1, pdata+p2);
					*PX_GameInstanceDataGetLocalPtr() = pdata[p1];
				}
				

			}
		}
	}
	
}

px_void PX_Object_BattleRender(px_surface* prendersurface, PX_Object* pObject, px_dword elapsed)
{
	px_float x, y, w, h;
	px_texture staticTex = { 0 };
	px_char content[256] = { 0 };
	PX_Object_Battle* pdesc = PX_ObjectGetDesc(PX_Object_Battle, pObject);
	PX_GameInstanceData* pdata= PX_GameInstanceDataGetLocalPtr();
	PX_OBJECT_INHERIT_CODE(pObject, x, y, w, h);
	if (pdata->binsize)
	{
		staticTex.height = 48;
		staticTex.limit_bottom = 47;
		staticTex.limit_right = 47;
		staticTex.MP = 0;
		staticTex.width = 48;
		staticTex.surfaceBuffer = (px_color*)PX_GameInstanceDataGetLocalPtr()->profile_data;
		PX_TextureRender(prendersurface, &staticTex, pdesc->renderSurface.width + 20, 24, PX_ALIGN_LEFTTOP, 0);
		
		PX_sprintf1(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.major"), PX_STRINGFORMAT_STRING(pdata->name));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24, PX_ALIGN_LEFTTOP,content,PX_OBJECT_UI_DEFAULT_FONTCOLOR);

		PX_sprintf1(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.author"), PX_STRINGFORMAT_STRING(pdata->author));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24+36, PX_ALIGN_LEFTTOP, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);

		PX_sprintf3(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.major info"), PX_STRINGFORMAT_INT(pdata->win), PX_STRINGFORMAT_INT(pdata->match), pdata->match?PX_STRINGFORMAT_FLOAT(pdata->win*100.f/pdata->match): PX_STRINGFORMAT_FLOAT(0));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24+72, PX_ALIGN_LEFTTOP, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	}
	else
	{
		PX_sprintf1(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.major"), PX_STRINGFORMAT_STRING(PX_JsonGetString(pdesc->language,"battle.unspecify")));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24 , PX_ALIGN_LEFTTOP, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);

		PX_sprintf1(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.author"), PX_STRINGFORMAT_STRING(PX_JsonGetString(pdesc->language, "battle.unspecify")));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24 + 32, PX_ALIGN_LEFTTOP, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);

		PX_sprintf3(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.major info"), PX_STRINGFORMAT_INT(0), PX_STRINGFORMAT_INT(0), PX_STRINGFORMAT_FLOAT(0));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24 + 76, PX_ALIGN_LEFTTOP, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	}
	
	if (pdesc->game.state==PX_GAME_STATE_RUN)
	{
		PX_sprintf2(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.vs"), PX_STRINGFORMAT_INT(pdesc->game.red), PX_STRINGFORMAT_INT(pdesc->game.blue));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24 + 128, PX_ALIGN_LEFTTOP, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);

		PX_sprintf1(content, sizeof(content), PX_JsonGetString(pdesc->language, "battle.step"), PX_STRINGFORMAT_INT((PX_OBJETC_GAME_TIME-pdesc->game.time)/1000));
		PX_FontModuleDrawText(prendersurface, pdesc->fm, pdesc->renderSurface.width + 20 + 100, 24 + 156, PX_ALIGN_LEFTTOP, content, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	}

	PX_SurfaceClear(&pdesc->renderSurface, 0, 0, pdesc->renderSurface.width, pdesc->renderSurface.height, PX_COLOR(255, 255, 255, 255));
	PX_GameRender(&pdesc->renderSurface, &pdesc->game, elapsed);
	PX_TextureCover(prendersurface, &pdesc->renderSurface, (px_int)10, (px_int)24, PX_ALIGN_LEFTTOP);
}

px_void PX_Object_BattleSliderBarOnChanged(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pBattleObject = (PX_Object*)ptr;
	PX_Object_Battle* pdesc = PX_ObjectGetDesc(PX_Object_Battle, pBattleObject);
	pdesc->gamespeed = (px_float)PX_Object_SliderBarGetValue(pObject);
}

px_void PX_Object_BattleOnObserverMode(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pBattleObject = (PX_Object*)ptr;
	PX_Object_Battle* pdesc = PX_ObjectGetDesc(PX_Object_Battle, pBattleObject);
	if (!pObject->Visible)
	{
		return;
	}
	if (PX_Object_Event_GetKeyDown(e)== PX_VK_F8)
	{
		pdesc->auto_observer_mode = !pdesc->auto_observer_mode;
		pdesc->auto_observer_elapsed = 0;
		if (pdesc->auto_observer_mode)
		{
			PX_Object_MessageBoxAlertOk(pdesc->messagebox, "Observer Mode On", 0, 0);
		}
		else
		{
			PX_Object_MessageBoxAlertOk(pdesc->messagebox, "Observer Mode Off", 0, 0);
		}
	}
	
}


PX_Object* PX_Object_BattleCreate(PX_Runtime* pruntime, PX_Object* parent, PX_Json* language, PX_Json* config, PX_FontModule* fm)
{
	PX_Object* pObject;
	PX_Object_Battle desc, * pdesc;

	PX_memset(&desc, 0, sizeof(desc));
	pObject = PX_ObjectCreateEx(&pruntime->mp_ui, parent, 0, 0, 0, 0, 0, 0, 0, PX_Object_BattleUpdate, PX_Object_BattleRender, 0, &desc, sizeof(desc));
	pdesc = PX_ObjectGetDesc(PX_Object_Battle, pObject);
	pdesc->fm = fm;
	pdesc->config = config;
	pdesc->language = language;
	pdesc->messagebox = PX_Object_MessageBoxCreate(&pruntime->mp_ui, pObject, fm);
	pdesc->gamespeed = 1;
	pdesc->pruntime = pruntime;
	pdesc->slider_speed = PX_Object_SliderBarCreate(&pruntime->mp_ui, pObject, PX_OBJECT_MAP_SIZE * PX_OBJECT_CELL_PIXELS + 20,
		24 + 220, pruntime->surface_width - PX_OBJECT_MAP_SIZE * PX_OBJECT_CELL_PIXELS - 30, 32, PX_OBJECT_SLIDERBAR_TYPE_HORIZONTAL, PX_OBJECT_SLIDERBAR_STYLE_BOX);

	PX_Object_SliderBarSetRange(pdesc->slider_speed, 1, 20);
	PX_ObjectRegisterEvent(pdesc->slider_speed, PX_OBJECT_EVENT_VALUECHANGED, PX_Object_BattleSliderBarOnChanged, pObject);
	PX_ObjectRegisterEvent(pObject, PX_OBJECT_EVENT_KEYDOWN, PX_Object_BattleOnObserverMode, pObject);

	pdesc->pList = PX_Object_ListCreate(&pruntime->mp_ui, pObject,
		PX_OBJECT_MAP_SIZE * PX_OBJECT_CELL_PIXELS + 20,
		24+256,
		pruntime->surface_width - PX_OBJECT_MAP_SIZE * PX_OBJECT_CELL_PIXELS - 30,
		pruntime->surface_height - 34 - 256,
		52,
		PX_Object_BattleListItemOnCreate,
		pObject
	);
	PX_ObjectRegisterEvent(pdesc->pList, PX_OBJECT_EVENT_VALUECHANGED, PX_Object_BattleListOnChanged,pObject);

	if (!PX_TextureCreate(&pruntime->mp_ui, &pdesc->renderSurface, PX_OBJECT_MAP_SIZE * PX_OBJECT_CELL_PIXELS, PX_OBJECT_MAP_SIZE * PX_OBJECT_CELL_PIXELS))
		return PX_NULL;

	PX_GameInitialize(pruntime,&pruntime->mp_game, &pdesc->game, pdesc->fm);
	return pObject;
}

px_void PX_Object_BattleRefresh(PX_Object* pObject)
{
	PX_Object_Battle* pdesc = PX_ObjectGetDesc(PX_Object_Battle, pObject);
	PX_GameInstanceData* pdata = PX_GameInstanceDataGetPtr();
	PX_GameInstanceData* plocaldata = PX_GameInstanceDataGetLocalPtr();
	px_int count = PX_GameInstanceDataGetCount();
	px_int i;
	PX_Object_ListClear(pdesc->pList);

	for (i = 0; i < count; i++)
	{
		PX_Object_ListAdd(pdesc->pList, pdata + i);
		//pdesc->result[i] = PX_GAME_RESULT_UNKNOW;
	}
	/*
	if (count>0&& plocaldata->binsize)
	{
		plocaldata->match = count;
		plocaldata->win = 0;
		for (i = 0; i < count; i++)
		{
			PX_GAME_RESULT result=PX_GameRunTest(&pdesc->pruntime->mp_game, plocaldata, pdata + i);
			if (result==PX_GAME_RESULT_REDWIN)
			{
				plocaldata->win ++;
			}
		}
	}
	*/
}

px_void PX_Object_BattleResetData(PX_Object* pObject)
{
	PX_Object_Battle* pdesc = PX_ObjectGetDesc(PX_Object_Battle, pObject);
	px_int count = PX_GameInstanceDataGetCount();
	px_int i;

	for (i = 0; i < count; i++)
	{
		pdesc->result[i] = PX_GAME_RESULT_UNKNOW;
	}
	PX_Object_GetList(pdesc->pList)->currentSelectedIndex = -1;
}