#include "PX_Object_ClientUpToDate.h"

px_void PX_Object_ClientUpToDateUpdate(PX_Object* pObject, px_dword elapsed)
{
	PX_Object_ClientUpToDate* pdesc = PX_ObjectGetDesc(PX_Object_ClientUpToDate, pObject);

	switch (pdesc->state)
	{
	case PX_OBJECT_CLIENTUPTODATE_STATE_QUERY:
		{
			PX_AES aes;
			
			if (pdesc->messagebox->Visible==PX_FALSE)
			{
				PX_Object_MessageBoxAlert(pdesc->messagebox, PX_JsonGetString(pdesc->language, "clientuptodate.connecting"));
			}

			if (pdesc->elapsed<=500)
			{
				pdesc->elapsed += elapsed;
				return;
			}

			if (!PX_TCPInitialize(&pdesc->tcp, PX_TCP_IP_TYPE_IPV4))
			{
				goto _ERROR;
			}

			if (!PX_TCPConnect(&pdesc->tcp, pdesc->serveraddr))
			{
				goto _ERROR;
			}

			//handshake
			do
			{
				px_byte input[16], output[16];
				px_dword ret[2];
				if (PX_TCPReceived(&pdesc->tcp, input, 16, 1800) != 16)
				{
					goto _ERROR;
				}

				if (!PX_memequ(pdesc->version,input,8))
				{
					PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "clientuptodate.version out of date"), 0, 0);
					goto _ERROR_NO_MSG;
				}
				

				PX_AES_Initialize(&aes, PX_AES_KeySize_Bits128, input);

				PX_memset(input, 0, sizeof(input));
				
				input[0] = 0;//opcode read
				

				PX_AES_Cipher(&aes, input, output);

				if (!PX_TCPSend(&pdesc->tcp, output, 16))
				{
					goto _ERROR;
				}
				if (PX_TCPReceived(&pdesc->tcp, ret, 8, 1800) != 8)
				{
					goto _ERROR;
				}

				if (ret[0]!=pdesc->count||ret[1]!=pdesc->local_crc32)
				{
					pdesc->count = ret[0];
					pdesc->current_pull_index = 0;
					pdesc->local_crc32 = ret[1];
					pdesc->state = PX_OBJECT_CLIENTUPTODATE_STATE_PULLING;
					PX_memset(PX_GameInstanceDataGetPtr(), 0, sizeof(PX_GameInstanceData) * PX_GAMEINSTANCEDATA_MAX);
				}
				else
				{
					pdesc->state = PX_OBJECT_CLIENTUPTODATE_STATE_DONE;
					PX_TCPFree(&pdesc->tcp);
				}

			} while (0);

		}
		break;
	case PX_OBJECT_CLIENTUPTODATE_STATE_PULLING:
		{
			PX_GameInstanceData* pdata = PX_GameInstanceDataGetPtr();
			if (pdesc->current_pull_index==pdesc->count)
			{
				px_dword end = 0xffff;
				pdesc->state = PX_OBJECT_CLIENTUPTODATE_STATE_DONE;
				PX_TCPSend(&pdesc->tcp, &end, 4);
				PX_TCPFree(&pdesc->tcp);
				PX_ObjectExecuteEvent(pObject->pParent, PX_OBJECT_BUILD_EVENT(PX_OBJECT_CLIENTUPTODATE_DONE));
				return;
			}
			else
			{
				px_char content[64];
				px_int retsize,offset;
				PX_GameInstanceData* pdata = PX_GameInstanceDataGetPtr();
				if (!PX_TCPSend(&pdesc->tcp, &pdesc->current_pull_index, 4))
				{
					pdesc->local_crc32 = 0;
					goto _ERROR;
				}
				offset=0;
				while (1)
				{
					px_byte buffer[sizeof(PX_GameInstanceData)];
					retsize=PX_TCPReceived(&pdesc->tcp, buffer, sizeof(PX_GameInstanceData), 2200);
					if (retsize)
					{
						PX_memcpy(((px_byte *)(pdata+ pdesc->current_pull_index))+offset,buffer,retsize);
						offset+=retsize;
						if (offset==sizeof(PX_GameInstanceData))
							break;

						if (retsize ==0)
						{
							pdesc->local_crc32 = 0;
							goto _ERROR;
						}
					} else
					{
						pdesc->local_crc32 = 0;
						goto _ERROR;
					}
				}
				pdesc->current_pull_index++;
				PX_sprintf2(content, sizeof(content), PX_JsonGetString(pdesc->language, "clientuptodate.synchronizing"), PX_STRINGFORMAT_INT(pdesc->current_pull_index), PX_STRINGFORMAT_INT(pdesc->count));
				PX_Object_MessageBoxAlert(pdesc->messagebox, content);
			}
		}
		break;
	case PX_OBJECT_CLIENTUPTODATE_STATE_DONE:
	default:
		{
			if (pdesc->messagebox->Visible)
			{
				if (PX_Object_GetMessageBox(pdesc->messagebox)->btn_Ok->Visible==PX_FALSE)
				{
					PX_Object_MessageBoxClose(pdesc->messagebox);
				}
			}
		}
		break;
	}
	return;
_ERROR:
	PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "clientuptodate.update failed"), 0, 0);
_ERROR_NO_MSG:
	
	pdesc->state = PX_OBJECT_CLIENTUPTODATE_STATE_DONE;
	PX_ObjectExecuteEvent(pObject->pParent, PX_OBJECT_BUILD_EVENT(PX_OBJECT_CLIENTUPTODATE_DONE));
	PX_TCPFree(&pdesc->tcp);
	return;
}


PX_Object* PX_Object_ClientUpToDateCreate(px_memorypool* mp, PX_Object* parent, PX_Json* language, PX_Json* config, PX_FontModule* fm)
{
	PX_Object* pObject;
	PX_Object_ClientUpToDate desc = { 0 }, * pdesc;
	desc.fm = fm;
	desc.language = language;
	desc.mp = mp;
	
	pObject=PX_ObjectCreateEx(mp, parent, 0, 0, 0, 0, 0, 0, 0, PX_Object_ClientUpToDateUpdate, 0, 0, &desc, sizeof(desc));
	pdesc = PX_ObjectGetDesc(PX_Object_ClientUpToDate, pObject);

	pdesc->messagebox = PX_Object_MessageBoxCreate(mp, pObject, fm);

	pdesc->serveraddr.ipv4 = PX_inet_addr(PX_JsonGetString(config, "server.ip"));
	if (pdesc->serveraddr.ipv4 == 0)
	{
		pdesc->serveraddr.ipv4 = PX_UDPGetHostByName(PX_JsonGetString(config, "server.host"));
		if (pdesc->serveraddr.ipv4 == 0)
		{
			return 0;
		}
	}
	pdesc->serveraddr.port = PX_htons((px_word)PX_JsonGetNumber(config, "server.listen"));

	pdesc->version[0] = 0;
	PX_strset(pdesc->version, PX_JsonGetString(config, "server.version"));
	return pObject;

}

px_void PX_Object_ClientUpToDateRefresh(PX_Object* pObject)
{
	PX_Object_ClientUpToDate* pdesc = PX_ObjectGetDesc(PX_Object_ClientUpToDate, pObject);
	pdesc->state = PX_OBJECT_CLIENTUPTODATE_STATE_QUERY;
	pdesc->elapsed = 0;
}

