#include "PX_Object_GameExplosion.h"
px_void PX_Object_GameExplosionRender(px_surface* psurface, PX_Object *pObject, px_dword elapsed)
{
	PX_Object_GameExplosion* pDesc = PX_ObjectGetDesc(PX_Object_GameExplosion, pObject);
	px_float objx, objy, objWidth, objHeight;
	px_float inheritX, inheritY;

	PX_ObjectGetInheritXY(pObject, &inheritX, &inheritY);

	objx = (pObject->x + inheritX);
	objy = (pObject->y + inheritY);
	objWidth = pObject->Width;
	objHeight = pObject->Height;

	pDesc->elapsed += elapsed;
	if (pDesc->elapsed > pDesc->alive)
	{
		PX_WorldRemoveObject(pDesc->pworld,pObject);
	}
	else
	{
		pDesc->launcher.LauncherInfo.position = PX_POINT(objx, objy, 0);
		PX_ParticalLauncherRender(psurface, &pDesc->launcher, elapsed);
	}
	
}


px_void PX_Object_GameExplosionFree(PX_Object *pObject)
{
	PX_Object_GameExplosion* pDesc = PX_ObjectGetDesc(PX_Object_GameExplosion, pObject);
	PX_ParticalLauncherFree(&pDesc->launcher);
	PX_TextureFree(&pDesc->texture);
}

PX_Object* PX_Object_GameExplosionCreate(px_memorypool* mp, PX_World* pworld, px_float x, px_float y, px_int count, px_float size, px_float power, px_int alive,px_dword type)
{
	PX_Object* pObject;
	PX_Object_GameExplosion *pdesc;
	PX_ParticalLauncher_InitializeInfo info;


	pObject= PX_ObjectCreateEx(mp, 0, x, y, 0, 0, 0, 0, PX_OBJECT_TYPE_EXPLOSIONX, 0, PX_Object_GameExplosionRender, PX_Object_GameExplosionFree, 0, sizeof(PX_Object_GameExplosion));
	pdesc = PX_ObjectGetDesc(PX_Object_GameExplosion, pObject);

	pdesc->mp = mp;
	pdesc->alive = alive;
	pdesc->power = power;
	pdesc->pworld = pworld;
	PX_TextureCreate(mp, &pdesc->texture, 32, 32);
	if (type==1)
	{
		PX_SurfaceClear(&pdesc->texture, 0, 0, 31, 31, PX_COLOR(96, 255, 0, 0));
		PX_GeoDrawBorder(&pdesc->texture, 0, 0, 31, 31, 3, PX_COLOR(192, 255, 0, 0));
	}
	else
	{
		PX_SurfaceClear(&pdesc->texture, 0, 0, 31, 31, PX_COLOR(96, 0, 0, 255));
		PX_GeoDrawBorder(&pdesc->texture, 0, 0, 31, 31, 3, PX_COLOR(192, 0, 0, 255));
	}
	
	pdesc->size = size;

	PX_ParticalLauncherInitializeDefaultInfo(&info);
	info.velocity = power * 100;
	info.deviation_velocity_max = 0;
	info.deviation_velocity_min = -80 * power;
	info.direction = PX_POINT(0, -1, 0);
	info.ak = 0.65f;
	info.alpha = 1;
	info.launchCount = count;
	info.generateDuration = 0;
	info.maxCount = count;
	info.alphaincrease = -1.0f / (alive / 1000.f);
	info.alive = alive;
	info.sizeincrease = 0.25f;
	info.rotation = 180;
	info.deviation_rotation = 90;
	info.deviation_atomsize_max = 0;
	info.deviation_atomsize_min = -0.8f;
	info.deviation_rangAngle = 360;
	info.deviation_position_distanceRange = size * 32;
	info.position = PX_POINT(x, y, 0);
	info.tex = &pdesc->texture;
	info.atomsize = size;
	PX_ParticalLauncherInitialize(&pdesc->launcher, mp, info);
	return pObject;
}

