#include "PX_Object_Join.h"


px_int APP_ExplorerGetPathFolderCount(const px_char* path, const char* filter)
{
	return PX_FileGetDirectoryFileCount(path, PX_FILEENUM_TYPE_FOLDER, filter);
}
px_int APP_ExplorerGetPathFileCount_C(const px_char* path, const char* filter)
{
	return PX_FileGetDirectoryFileCount(path, PX_FILEENUM_TYPE_FILE, "c\0C\0txt\0TXT");
}
px_int APP_ExplorerGetPathFileCount_PNG(const px_char* path, const char* filter)
{
	return PX_FileGetDirectoryFileCount(path, PX_FILEENUM_TYPE_FILE, "png\0PNG\0Png\0");
}


px_int APP_ExplorerGetPathFolderName(const char path[], int count, char FileName[][260], const char* filter)
{
	return PX_FileGetDirectoryFileName(path, count, FileName, PX_FILEENUM_TYPE_FOLDER, filter);
}

px_int APP_ExplorerGetPathFileName_C(const char path[], int count, char FileName[][260], const char* filter)
{
	return PX_FileGetDirectoryFileName(path, count, FileName, PX_FILEENUM_TYPE_FILE, "c\0C\0");
}
px_int APP_ExplorerGetPathFileName_PNG(const char path[], int count, char FileName[][260], const char* filter)
{
	return PX_FileGetDirectoryFileName(path, count, FileName, PX_FILEENUM_TYPE_FILE, "png\0PNG\0Png\0");
}


px_void PX_Object_ButtonOnExpolrerC(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pJoinObject = (PX_Object*)ptr;
	PX_Object_Join* pdesc = PX_ObjectGetDesc(PX_Object_Join, pJoinObject);
	PX_Object_ExplorerOpen(pdesc->explorer_c);
}

px_void PX_Object_ButtonOnExpolrerPng(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pJoinObject = (PX_Object*)ptr;
	PX_Object_Join* pdesc = PX_ObjectGetDesc(PX_Object_Join, pJoinObject);
	if (PX_ObjectIsCursorInRegion(pObject,e))
	{
		PX_Object_ExplorerOpen(pdesc->explorer_png);
	}
}


px_void PX_Object_ExpolorerOnSelectedC(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pJoinObject = (PX_Object*)ptr;
	PX_Object_Join* pdesc = PX_ObjectGetDesc(PX_Object_Join, pJoinObject);
	px_byte cache[1024 * 512];
	px_memory bin;
	px_memorypool mp = MP_Create(cache, sizeof(cache));
	PX_Compiler compiler;
	PX_SCRIPT_ASM_HEADER* pheader;
	px_char path[260];
	PX_IO_Data data;
	PX_Object_ExplorerGetPath(pObject, path, 0);
	data = PX_LoadFileToIOData(path);
	if (data.size==0)
	{
		return;
	}
	
	if (data.size>= sizeof(((PX_GameInstanceData*)0)->code))
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "size too large(>32kb)"), 0, 0);
		goto _END;
	}

	printf("code size:%d bytes\n", data.size);

	PX_StringClear(&pdesc->code);
	PX_MemoryClear(&pdesc->bin);

	PX_StringCat(&pdesc->code, (const px_char *)data.buffer);
	PX_FreeIOData(&data);
	
	PX_MemoryInitialize(&mp, &bin);
	PX_CompilerInitialize(&mp, &compiler);
	PX_CompilerAddSource(&compiler, PX_Object_RobotGetStdlib());
	PX_CompilerAddSource(&compiler, pdesc->code.buffer);
	
	
	if (!PX_CompilerCompile(&compiler, &bin, 0, "main"))
	{
		//px_char *perr=PX_CompilerError(&compiler);
		//printf("Compiler Error near:\n");
		//printf((const px_char*)perr);
		//printf("\n");
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.compile failed"), 0, 0);
		goto _END;
	}

	printf("bin size:%d bytes\n", bin.usedsize);

	pheader = (PX_SCRIPT_ASM_HEADER*)bin.buffer;
	if (pheader->stringSize!= 0||pheader->memsize!=0)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.painterscript limit"), 0, 0);
		goto _END;
	}

	if (pheader->globalsize > PX_OBJECT_ROBOT_GLOBAL_SIZE)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.heap overflow"), 0, 0);
		goto _END;
	}
	
	if (pheader->stacksize > PX_OBJECT_ROBOT_STACK_SIZE)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.stack overflow"), 0, 0);
		goto _END;
	}

	if (bin.usedsize > sizeof(((PX_GameInstanceData *)0)->bin))
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.size too large"), 0, 0);
		goto _END;
	}

	
	PX_MemoryCopy(&pdesc->bin, bin.buffer, 0, bin.usedsize);
	PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.compile done"), 0, 0);

	do 
	{
		px_char name[96];
		px_char content[128];
		PX_FileGetName(path, name, sizeof(name));
		PX_sprintf1(content, sizeof(content), PX_JsonGetString(pdesc->language, "join.code loaded"), PX_STRINGFORMAT_STRING(name));
		PX_Object_PushButtonSetText(PX_Object_GetCursorButtonPushButton(pdesc->button_load), content);
	} while (0);

	_END:
		return;

}

px_void PX_Object_ExpolorerOnSelectedPNG(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_texture tex;
	PX_Object* pJoinObject = (PX_Object*)ptr;
	PX_Object_Join* pdesc = PX_ObjectGetDesc(PX_Object_Join, pJoinObject);
	px_char path[260];
	PX_Object_ExplorerGetPath(pObject, path, 0);
	if (PX_LoadTextureFromFile(pdesc->mp, &tex, path))
	{
		PX_TextureFree(&pdesc->profile_photo);
		PX_TextureFree(&pdesc->profile_photo48x48);
		PX_TextureCreateScale(pdesc->mp, &tex, 128, 128, &pdesc->profile_photo);
		PX_TextureCreateScale(pdesc->mp, &tex, 48, 48, &pdesc->profile_photo48x48);
		PX_TextureFree(&tex);
	}

}

px_void PX_Object_ButtonOnPublish(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pJoinObject = (PX_Object*)ptr;
	PX_Object_Join* pdesc = PX_ObjectGetDesc(PX_Object_Join, pJoinObject);
	
	PX_TCP tcp;
	PX_AES aes;

	if (PX_strlen(PX_Object_VariousGetText(pdesc->Edit_Name))> PX_GAMEINSTANCEDATA_CONTENT_SIZE-1|| PX_strlen(PX_Object_VariousGetText(pdesc->Edit_Name))<1)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.invalid name"), 0, 0);
		return;
	}
	
	if (PX_strlen(PX_Object_VariousGetText(pdesc->Edit_Author)) > PX_GAMEINSTANCEDATA_CONTENT_SIZE - 1 || PX_strlen(PX_Object_VariousGetText(pdesc->Edit_Author)) < 1)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.invalid author"), 0, 0);
		return;
	}

	if (PX_strlen(PX_Object_VariousGetText(pdesc->Edit_key)) > PX_GAMEINSTANCEDATA_CONTENT_SIZE - 1 || PX_strlen(PX_Object_VariousGetText(pdesc->Edit_key))<=0)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.need key"), 0, 0);
		return;
	}

	if (pdesc->bin.usedsize==0)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.invalid code"), 0, 0);
		return;
	}

	if (!PX_TCPInitialize(&tcp, PX_TCP_IP_TYPE_IPV4))
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.connect failed"), 0, 0);
		return;
	}

	if (!PX_TCPConnect(&tcp, pdesc->serveraddr))
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.connect failed"), 0, 0);
		goto _ERROR;
	}

	//handshake
	do 
	{
		px_byte input[16], output[16];
		px_dword ret;
		if (PX_TCPReceived(&tcp, input, 16, 1800) != 16)
		{
			goto _ERROR;
		}

		PX_AES_Initialize(&aes, PX_AES_KeySize_Bits128, input);

		PX_memset(input, 0, sizeof(input));
		input[0] = 1;//opcode

		PX_AES_Cipher(&aes, input, output);

		if (!PX_TCPSend(&tcp, output, 16))
		{
			goto _ERROR;
		}
		if (PX_TCPReceived(&tcp, &ret, 4, 1800) != 4)
		{
			goto _ERROR;
		}
		if (ret != 0)
		{
			goto _ERROR;
		}
	} while (0);


	//data
	do 
	{
		px_dword ret;
		PX_GameInstanceData data = { 0 };
		PX_strcpy(data.author,  PX_Object_VariousGetText(pdesc->Edit_Author), sizeof(data.author));
		PX_strcpy(data.name, PX_Object_VariousGetText(pdesc->Edit_Name), sizeof(data.name));
		PX_strcpy(data.key,  PX_Object_VariousGetText(pdesc->Edit_key), sizeof(data.key));
		PX_strcpy((px_char *)data.code,  (const px_char *)pdesc->code.buffer, sizeof(data.code));
		PX_memcpy(data.profile_data, pdesc->profile_photo48x48.surfaceBuffer, sizeof(data.profile_data));
		if (!PX_TCPSend(&tcp, &data, sizeof(data)))
		{
			goto _ERROR;
		}
		if (PX_TCPReceived(&tcp, &ret, 4, 1800) != 4)
		{
			goto _ERROR;
		}

		if (ret==1)
		{
			PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.invalid key"), 0, 0);
			return;
		}
		if (ret != 0)
		{
			goto _ERROR;
		}
	} while (0);

	PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.publish succeeded"), 0, 0);
	PX_TCPFree(&tcp);
	return;

_ERROR:
	PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.connect failed"), 0, 0);
	PX_TCPFree(&tcp);
	return;

}

px_void PX_Object_ButtonOnTest(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pJoinObject = (PX_Object*)ptr;
	PX_Object_Join* pdesc = PX_ObjectGetDesc(PX_Object_Join, pJoinObject);

	if (PX_strlen(PX_Object_VariousGetText(pdesc->Edit_Name)) > PX_GAMEINSTANCEDATA_CONTENT_SIZE - 1 || PX_strlen(PX_Object_VariousGetText(pdesc->Edit_Name)) < 1)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.invalid name"), 0, 0);
		return;
	}


	if (pdesc->bin.usedsize == 0)
	{
		PX_Object_MessageBoxAlertOk(pdesc->messagebox, PX_JsonGetString(pdesc->language, "join.invalid code"), 0, 0);
		return;
	}

	//data
	do
	{
		PX_GameInstanceData data = { 0 };
		PX_strcpy(data.author, PX_Object_VariousGetText(pdesc->Edit_Author), sizeof(data.author));
		PX_strcpy(data.name, PX_Object_VariousGetText(pdesc->Edit_Name), sizeof(data.name));
		PX_strcpy(data.key, PX_Object_VariousGetText(pdesc->Edit_key), sizeof(data.key));
		PX_strcpy((px_char *)data.code, (const px_char *)pdesc->code.buffer, sizeof(data.code));
		PX_memcpy(data.bin, pdesc->bin.buffer, pdesc->bin.usedsize);
		data.binsize = pdesc->bin.usedsize;
		PX_memcpy(data.profile_data, pdesc->profile_photo48x48.surfaceBuffer, sizeof(data.profile_data));
		*PX_GameInstanceDataGetLocalPtr() = data;

	} while (0);

	PX_ObjectExecuteEvent(pJoinObject->pParent, PX_OBJECT_BUILD_EVENT(PX_OBJECT_JOIN_EVENT_TEST));
	
	return;


}

px_void PX_Object_JoinRender(px_surface* psurface, PX_Object* pobject, px_dword elapsed)
{
	PX_Object_Join* pDesc = PX_ObjectGetDesc(PX_Object_Join, pobject);
	pDesc->Image_profile->x = pobject->Width / 2 - pDesc->Image_profile->Width / 2;
	pDesc->Image_profile->y = 0;

	pDesc->Edit_Name->x = pobject->Width / 2 - pDesc->Edit_Name->Width / 2-32;
	pDesc->Edit_Name->y = pDesc->Image_profile->Height + 12;

	pDesc->Edit_Author->x = pobject->Width / 2 - pDesc->Edit_Author->Width / 2-32;
	pDesc->Edit_Author->y = pDesc->Edit_Name->y+ pDesc->Edit_Name->Height + 12;

	pDesc->Edit_key->x = pobject->Width / 2 - pDesc->Edit_key->Width / 2-32;
	pDesc->Edit_key->y = pDesc->Edit_Author->y + pDesc->Edit_Author->Height + 12;

	pDesc->button_load->x = pobject->Width / 2 - pDesc->button_load->Width/2-32;
	pDesc->button_load->y = pDesc->Edit_key->y + pDesc->Edit_key->Height + 12;

	pDesc->button_simulate->x = pobject->Width / 2 - pDesc->button_simulate->Width -16;
	pDesc->button_simulate->y = pDesc->button_load->y + pDesc->button_load->Height + 12;

	pDesc->button_join->x = pobject->Width / 2 + 16;
	pDesc->button_join->y = pDesc->button_load->y + pDesc->button_load->Height + 12;

}


PX_Object* PX_Object_JoinCreate(px_memorypool* mp, PX_Object* parent, px_float x, px_float y, PX_Json* language, PX_Json* config, PX_FontModule* fm)
{
	PX_Object* pObject;
	PX_Object_Join desc,*pdesc;
	PX_memset(&desc, 0, sizeof(desc));
	
	
	pObject = PX_ObjectCreateEx(mp, parent, x, y, 328, 512, 0, 0, 0, 0, PX_Object_JoinRender, 0, &desc, sizeof(desc));
	pdesc = PX_ObjectGetDesc(PX_Object_Join, pObject);

	

	pdesc->fm = fm;
	pdesc->language = language;
	pdesc->config = config;
	pdesc->mp = mp;

	pdesc->serveraddr.ipv4 = PX_inet_addr(PX_JsonGetString(config, "server.ip"));
	if (pdesc->serveraddr.ipv4==0)
	{
		pdesc->serveraddr.ipv4 = PX_UDPGetHostByName(PX_JsonGetString(config, "server.host"));
		if (pdesc->serveraddr.ipv4 == 0)
		{
			return 0;
		}
	}
	
	pdesc->serveraddr.port = PX_htons((px_word)PX_JsonGetNumber(config, "server.listen"));

	PX_StringInitialize(mp, &pdesc->code);
	PX_MemoryInitialize(mp, &pdesc->bin);

	PX_LoadTextureFromFile(pdesc->mp, &pdesc->profile_photo, "assets/gear.png");
	PX_TextureCreateScale(pdesc->mp, &pdesc->profile_photo, 48, 48, &pdesc->profile_photo48x48);

	pdesc->Image_profile = PX_Object_ImageCreate(mp, pObject, 0, 0, 128, 128, &pdesc->profile_photo);
	PX_ObjectRegisterEvent(pdesc->Image_profile, PX_OBJECT_EVENT_CURSORDOWN, PX_Object_ButtonOnExpolrerPng, pObject);

	pdesc->Edit_Name = PX_Object_VariousCreate(mp, pObject, 0, 0, 328, 24,PX_JsonGetString(language,"join.name"), PX_OBJECT_VARIOUS_TYPE_EDIT, fm);
	PX_Object_VariousEditSetMaxLength(pdesc->Edit_Name, 30);
	pdesc->Edit_Author = PX_Object_VariousCreate(mp, pObject, 0, 0, 328, 24, PX_JsonGetString(language, "join.author"), PX_OBJECT_VARIOUS_TYPE_EDIT, fm);
	PX_Object_VariousEditSetMaxLength(pdesc->Edit_Author, 30);
	pdesc->Edit_key = PX_Object_VariousCreate(mp, pObject, 0, 0, 328, 24, PX_JsonGetString(language, "join.key"), PX_OBJECT_VARIOUS_TYPE_EDIT, fm);
	PX_Object_VariousEditSetMaxLength(pdesc->Edit_key, 24);

	pdesc->button_load= PX_Object_CursorButtonCreate(mp, pObject, 0, 0, 328, 32, PX_JsonGetString(language, "join.load"), fm, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	PX_ObjectRegisterEvent(PX_Object_GetCursorButtonPushButton(pdesc->button_load), PX_OBJECT_EVENT_EXECUTE, PX_Object_ButtonOnExpolrerC, pObject);

	pdesc->messagebox = PX_Object_MessageBoxCreate(mp, pObject, fm);

	pdesc->explorer_c = PX_Object_ExplorerCreate(mp, pObject, -128, -64, 800, 600, fm,
		APP_ExplorerGetPathFolderCount,
		APP_ExplorerGetPathFileCount_C,
		APP_ExplorerGetPathFolderName,
		APP_ExplorerGetPathFileName_C,
		""
	);

	pdesc->explorer_png = PX_Object_ExplorerCreate(mp, pObject, -128, -64, 800, 600, fm,
		APP_ExplorerGetPathFolderCount,
		APP_ExplorerGetPathFileCount_PNG,
		APP_ExplorerGetPathFolderName,
		APP_ExplorerGetPathFileName_PNG,
		""
	);

	PX_ObjectRegisterEvent(pdesc->explorer_c, PX_OBJECT_EVENT_EXECUTE, PX_Object_ExpolorerOnSelectedC, pObject);
	PX_ObjectRegisterEvent(pdesc->explorer_png, PX_OBJECT_EVENT_EXECUTE, PX_Object_ExpolorerOnSelectedPNG, pObject);

	pdesc->button_simulate = PX_Object_CursorButtonCreate(mp, pObject, 0, 0, 128, 32, PX_JsonGetString(language, "join.simulate"), fm, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	pdesc->button_join = PX_Object_CursorButtonCreate(mp, pObject, 0, 0, 128, 32, PX_JsonGetString(language, "join.join"), fm, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	PX_ObjectRegisterEvent(PX_Object_GetCursorButtonPushButton(pdesc->button_join), PX_OBJECT_EVENT_EXECUTE, PX_Object_ButtonOnPublish, pObject);
	PX_ObjectRegisterEvent(PX_Object_GetCursorButtonPushButton(pdesc->button_simulate), PX_OBJECT_EVENT_EXECUTE, PX_Object_ButtonOnTest, pObject);
	return pObject;
}

