#include "PX_Object_Missile.h"

void PX_Object_MissileUpdate(PX_Object *pObject, px_dword elapsed)
{
	px_float x, y, w, h;
	px_int i;
	px_dword updateam = 50;
	PX_Object_Missile* pdesc= PX_ObjectGetDesc(PX_Object_Missile, pObject);

	PX_OBJECT_INHERIT_CODE(pObject, x, y, w, h);
	pdesc->elapsed += elapsed;

	if (pdesc->elapsed > pdesc->alive)
	{
		PX_WorldRemoveObject(pdesc->pworld, pObject);
		return;
	}

	if (x<-512||x>PX_OBJECT_MAP_SIZE* PX_OBJECT_CELL_PIXELS+512)
	{
		PX_WorldRemoveObject(pdesc->pworld, pObject);
		return;
	}

	if (y<-512 || y>PX_OBJECT_MAP_SIZE * PX_OBJECT_CELL_PIXELS+512)
	{
		PX_WorldRemoveObject(pdesc->pworld, pObject);
		return;
	}

	pObject->x += pdesc->vx * elapsed / 1000;
	pObject->y += pdesc->vy * elapsed / 1000;

	

	pdesc->delay += elapsed;
	while (pdesc->delay > updateam)
	{
		pdesc->delay -= updateam;
		pdesc->array[0].x = pObject->x;
		pdesc->array[0].y = pObject->y;

		for (i = PX_COUNTOF(pdesc->array) - 1; i > 0; i--)
		{
			pdesc->array[i] = pdesc->array[i - 1];
		}
	}


	


}

px_void PX_Object_MissileRender(px_surface* psurface, PX_Object* pObject,  px_dword elapsed)
{
	px_int i;
	px_float scale;
	px_color color;
	PX_Object_Missile* pdesc = PX_ObjectGetDesc(PX_Object_Missile, pObject);


	for (i = PX_COUNTOF(pdesc->array) - 1; i >= 0; i--)
	{
		px_float a = 1.0f- pdesc->elapsed/ (1.0f * pdesc->alive);
		color = pdesc->clr;
		color._argb.a = (px_uchar)(color._argb.a * a );
		scale = 1.0f-i / (1.0f*PX_COUNTOF(pdesc->array))  ;

		color._argb.a = (px_uchar)(color._argb.a * scale);
		color._argb.a = (px_uchar)(color._argb.a * scale);
		PX_GeoDrawSolidCircle(psurface, (px_int)pdesc->array[i].x, (px_int)pdesc->array[i].y, (px_int)(pdesc->size * scale), color);
	}
}

px_void PX_Object_MissileImpact(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Object* pTargetObject = PX_Object_Event_GetImpactTargetObject(e);
	PX_Object_Missile *pdesc= PX_ObjectGetDesc(PX_Object_Missile, pObject);
	e = PX_OBJECT_BUILD_EVENT(PX_OBJECT_EVENT_IMPACT);
	PX_Object_Event_SetImpactTargetObject(&e, pObject);
	PX_ObjectExecuteEvent(pTargetObject, e);
	PX_WorldRemoveObject(pdesc->pworld, pObject);
}

PX_Object* PX_Object_MissileCreate(px_memorypool* mp, PX_World* pworld, px_float x, px_float y, px_float vx, px_float vy, px_dword target_type, px_color color)
{
	PX_Object_Missile Desc,*pDesc;
	PX_Object* pObject;


	px_int i;
	PX_memset(&Desc, 0, sizeof(Desc));
	Desc.clr = color;
	Desc.vx = vx;
	Desc.vy = vy;
	Desc.size = 6;
	Desc.pworld = pworld;
	Desc.alive = 8000;
	pObject = PX_ObjectCreateEx(mp, 0, x, y, 0, 8, 8, 0, 0, PX_Object_MissileUpdate, PX_Object_MissileRender, 0, &Desc, sizeof(Desc));
	
	pDesc = PX_ObjectGetDesc(PX_Object_Missile, pObject);
	for (i = 0; i < PX_COUNTOF(pDesc->array); i++)
	{
		pDesc->array[i] = PX_POINT(x, y, 0);
	}
	
	pObject->impact_target_type = target_type;

	PX_ObjectRegisterEvent(pObject, PX_OBJECT_EVENT_IMPACT, PX_Object_MissileImpact,pObject);
	return pObject;
}

