#include "PX_Object_Robot.h"

static const px_char stdlib[] = "#name \"stdlib.h\"\r\n\
host void MoveLeft();\r\n\
host void MoveRight();\r\n\
host void MoveUp();\r\n\
host void MoveDown();\r\n\
host int EnemyX();\r\n\
host int EnemyY();\r\n\
host int MyX();\r\n\
host int MyY(); \r\n\
host int Map(int x, int y);\r\n\
host int Fire();\r\n\
host int Rand();";


px_void PX_Object_RobotFixPosition(PX_Object* pObject)
{
	pObject->x = (px_int)(pObject->x / PX_OBJECT_CELL_PIXELS) * PX_OBJECT_CELL_PIXELS + PX_OBJECT_CELL_PIXELS / 2.f;
	pObject->y = (px_int)(pObject->y / PX_OBJECT_CELL_PIXELS) * PX_OBJECT_CELL_PIXELS + PX_OBJECT_CELL_PIXELS / 2.f;
}

px_bool PX_Object_Robot_VmMoveLeft(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	
	if (pdesc->type== PX_OBJECT_ROBOT_TYPE_RED)
	{
		pdesc->velocity.x = -PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.y = 0;
		pdesc->direction = PX_POINT2D(-1, 0);
	}
	else
	{
		pdesc->velocity.x = PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.y = 0;
		pdesc->direction = PX_POINT2D(1, 0);
	}
	PX_Object_RobotFixPosition(pObject);
	PX_VM_Sleep(Ins, 1000);

	return PX_TRUE;
}

px_bool PX_Object_Robot_VmMoveRight(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);

	if (pdesc->type == PX_OBJECT_ROBOT_TYPE_RED)
	{
		pdesc->velocity.x = PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.y = 0;
		pdesc->direction = PX_POINT2D(1, 0);
	}
	else
	{
		pdesc->velocity.x = -PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.y = 0;
		pdesc->direction = PX_POINT2D(-1, 0);
	}
	PX_Object_RobotFixPosition(pObject);
	PX_VM_Sleep(Ins, 1000);

	return PX_TRUE;
}

px_bool PX_Object_Robot_VmMoveUp(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	if (pdesc->type==PX_OBJECT_ROBOT_TYPE_RED)
	{
		pdesc->velocity.y = -PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.x = 0;
		pdesc->direction = PX_POINT2D(0, -1);
	}
	else
	{
		pdesc->direction = PX_POINT2D(0, 1);
		pdesc->velocity.y = PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.x = 0;
	}
	PX_Object_RobotFixPosition(pObject);
	PX_VM_Sleep(Ins, 1000);
	return PX_TRUE;
}

px_bool PX_Object_Robot_VmMoveDown(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	if (pdesc->type == PX_OBJECT_ROBOT_TYPE_RED)
	{
		pdesc->direction = PX_POINT2D(0, 1);
		pdesc->velocity.y = PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.x = 0;
	}
	else
	{
		pdesc->direction = PX_POINT2D(0, -1);
		pdesc->velocity.y = -PX_OBJECT_CELL_PIXELS;
		pdesc->velocity.x = 0;
	}
	PX_Object_RobotFixPosition(pObject);
	PX_VM_Sleep(Ins, 1000);
	return PX_TRUE;
}

px_bool PX_Object_Robot_VmFire(PX_VM* Ins, px_void* userptr)
{
	px_float x, y, w, h;
	PX_Object* pObject = (PX_Object*)userptr,*pMissile;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	px_dword target_type = pdesc->type == 1 ? 2 : 1;
	px_color clr= pdesc->type == 1 ? PX_COLOR_RED : PX_COLOR_BLUE;
	PX_OBJECT_INHERIT_CODE(pObject, x, y, w, h);
	
	pMissile=PX_Object_MissileCreate(pdesc->mp, pdesc->pworld, x, y, PX_OBJECT_ROBOT_MISSILE_SPEED * pdesc->direction.x, PX_OBJECT_ROBOT_MISSILE_SPEED * pdesc->direction.y, target_type, clr);
	PX_WorldAddObject(pdesc->pworld, pMissile);
	pdesc->launch_elapsed = 0;
	pdesc->velocity.y = 0;
	pdesc->velocity.x = 0;
	if (pdesc->pruntime)
		PX_SoundPlayAdd(&pdesc->pruntime->soundplay, PX_SoundCreate(PX_ResourceLibraryGetSound(&pdesc->pruntime->ResourceLibrary, "fire"), 0));
	PX_VM_Sleep(Ins, 1000);
	return PX_TRUE;
}

px_bool PX_Object_Robot_VmGetEnemyPosX(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	PX_Object *pRobotObjects;
	
	px_dword impact_type=pdesc->type==1?2:1;

	if (PX_WorldSearch(pdesc->pworld,&pRobotObjects,1, impact_type))
	{
		px_int x;
		x = (px_int)(pRobotObjects->x / PX_OBJECT_CELL_PIXELS);

		if (pdesc->type == PX_OBJECT_ROBOT_TYPE_RED)
		{
			PX_VM_RET_int(Ins, x);
		}
		else
		{
			PX_VM_RET_int(Ins, PX_OBJECT_MAP_SIZE - x-1);
		}
	}
	else
	{
		PX_VM_RET_int(Ins, 0);
	}

	return PX_TRUE;
}
px_bool PX_Object_Robot_VmGetEnemyPosY(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	PX_Object *pRobotObjects;
	px_dword impact_type = pdesc->type == 1 ? 2 : 1;
	if (PX_WorldSearch(pdesc->pworld, &pRobotObjects, 1, impact_type))
	{
		px_int y;
		y = (px_int)(pRobotObjects->y / PX_OBJECT_CELL_PIXELS);

		if (pdesc->type == PX_OBJECT_ROBOT_TYPE_RED)
		{
			PX_VM_RET_int(Ins, y);
		}
		else
		{
			PX_VM_RET_int(Ins, PX_OBJECT_MAP_SIZE - y-1);
		}
	}
	else
	{
		PX_VM_RET_int(Ins, 0);
	}

	return PX_TRUE;
}

px_bool PX_Object_Robot_VmGetMyPosX(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	px_int x = (px_int)(pObject->x / PX_OBJECT_CELL_PIXELS);
	if (pdesc->type == PX_OBJECT_ROBOT_TYPE_RED)
	{
		PX_VM_RET_int(Ins, x);
	}
	else
	{
		PX_VM_RET_int(Ins, PX_OBJECT_MAP_SIZE - x-1);
	}

	return PX_TRUE;
}

px_bool PX_Object_Robot_VmGetMyPosY(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	px_int y = (px_int)(pObject->y / PX_OBJECT_CELL_PIXELS);
	if (pdesc->type == PX_OBJECT_ROBOT_TYPE_RED)
	{
		PX_VM_RET_int(Ins, y);
	}
	else
	{
		PX_VM_RET_int(Ins, PX_OBJECT_MAP_SIZE - y-1);
	}

	return PX_TRUE;
}

px_bool PX_Object_Robot_VmGetMap(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	px_int x, y, type;
	if (PX_VM_HOSTPARAM(Ins, 0).type != PX_VM_VARIABLE_TYPE_INT)
	{
		return PX_FALSE;
	}
	if (PX_VM_HOSTPARAM(Ins, 1).type != PX_VM_VARIABLE_TYPE_INT)
	{
		return PX_FALSE;
	}
	x = PX_VM_HOSTPARAM(Ins, 0)._int;
	y = PX_VM_HOSTPARAM(Ins, 1)._int;

	if (pdesc->type == PX_OBJECT_ROBOT_TYPE_BLUE)
	{
		x = PX_OBJECT_MAP_SIZE - x-1;
		y = PX_OBJECT_MAP_SIZE - y-1;
	}

	type = (px_int)PX_Object_MapGetCell(pdesc->map, x, y);
	if (type == PX_OBJECT_MAP_CELL_NONE)
	{
		PX_VM_RET_int(Ins, 1);
	}
	else if (type == pdesc->type)
	{
		PX_VM_RET_int(Ins, 2);
	}
	else
	{
		PX_VM_RET_int(Ins, 0);
	}
	return PX_TRUE;
}

px_bool PX_Object_Robot_VmRand(PX_VM* Ins, px_void* userptr)
{
	PX_Object* pObject = (PX_Object*)userptr;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	pdesc->vm_seed = PX_randEx(pdesc->vm_seed);
	PX_VM_RET_int(Ins, (px_int)((pdesc->vm_seed/10)%10001));
	return PX_TRUE;
}

px_void PX_Object_RobotUpdate(PX_Object* pObject, px_dword elapsed)
{
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	px_int ix, iy,type;
	
	if (pdesc->death)
	{
		return;
	}

	

	pdesc->vm_elapsed += elapsed;
	do
	{
		
		//10khz ticks for VM 
		if (PX_VMIsRuning(&pdesc->vm))
		{
			PX_VMRun(&pdesc->vm, 1024 * 10, elapsed);
			if (pdesc->vm.pThread[0].sleep == 0)
			{
				PX_VMSleep(&pdesc->vm, 0, 1000);
			}
		}
		else
		{

			PX_VMBeginThreadFunction(&pdesc->vm, 0, "main", 0, 0);
			PX_VMRun(&pdesc->vm, 1024 * 10, elapsed);
			if (pdesc->vm.pThread[0].sleep==0)
			{
				PX_VMSleep(&pdesc->vm,0, 1000);
			}

		}
	} while (0);
	

	//<--x
	iy = (px_int)(pObject->y)/ PX_OBJECT_CELL_PIXELS;
	ix = (px_int)((pObject->x - PX_OBJECT_CELL_PIXELS / 2-0.01f)/ PX_OBJECT_CELL_PIXELS);
	if (pObject->x - pObject->Width / 2 < 0)
	{
		if (pdesc->velocity.x < 0)
			pdesc->velocity.x = 0;
	}
	else
	{
		type = (px_int)PX_Object_MapGetCell(pdesc->map, ix, iy);
		if (type != 0 && type != pdesc->type)
		{
			if (pdesc->velocity.x < 0)
				pdesc->velocity.x = 0;
		}
	}
	

	//x-->
	iy = (px_int)(pObject->y) / PX_OBJECT_CELL_PIXELS;
	ix = (px_int)((pObject->x + PX_OBJECT_CELL_PIXELS / 2 + 0.01f) / PX_OBJECT_CELL_PIXELS);
	if (ix >= PX_OBJECT_MAP_SIZE)
	{
		if(pdesc->velocity.x>0)
			pdesc->velocity.x = 0;
	}
	else
	{
		type = (px_int)PX_Object_MapGetCell(pdesc->map, ix, iy);

		if (type != 0 && type != pdesc->type)
		{
			if (pdesc->velocity.x > 0)
				pdesc->velocity.x = 0;
		}
	}
	
	
	//^y
	iy = (px_int)(pObject->y- PX_OBJECT_CELL_PIXELS /2- 0.01f) / PX_OBJECT_CELL_PIXELS;
	ix = (px_int)(pObject->x  / PX_OBJECT_CELL_PIXELS);
	if (pObject->y - pObject->Height / 2 < 0)
	{
		if (pdesc->velocity.y < 0)
			pdesc->velocity.y = 0;
	}
	else
	{
		type = (px_int)PX_Object_MapGetCell(pdesc->map, ix, iy);

		if (type != 0 && type != pdesc->type)
		{
			if (pdesc->velocity.y < 0)
				pdesc->velocity.y = 0;
		}
	}
	
	//|y
	iy = (px_int)(pObject->y + PX_OBJECT_CELL_PIXELS / 2 + 0.01f) / PX_OBJECT_CELL_PIXELS;
	ix = (px_int)(pObject->x / PX_OBJECT_CELL_PIXELS);
	if (iy>= PX_OBJECT_MAP_SIZE)
	{
		if (pdesc->velocity.y > 0)
			pdesc->velocity.y = 0;
	}
	else
	{
		type = (px_int)PX_Object_MapGetCell(pdesc->map, ix, iy);

		if (type != 0 && type != pdesc->type)
		{
			if (pdesc->velocity.y > 0)
				pdesc->velocity.y = 0;
		}
	}

	pObject->x += pdesc->velocity.x * elapsed / 1000.f;
	pObject->y += pdesc->velocity.y * elapsed / 1000.f;
	iy = (px_int)(pObject->y  / PX_OBJECT_CELL_PIXELS);
	ix = (px_int)(pObject->x / PX_OBJECT_CELL_PIXELS);

	PX_Object_MapMark(pdesc->map, ix, iy, pdesc->type);
}

px_void PX_Object_RobotRender(px_surface* prendersurface, PX_Object* pObject, px_dword elapsed)
{
	px_float x, y, w, h;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	PX_OBJECT_INHERIT_CODE(pObject, x, y, w, h);
	if (pdesc->death)
	{
		return;
	}

	pdesc->launch_elapsed += elapsed;
	if (pdesc->launch_elapsed<500)
	{
		px_float a = pdesc->launch_elapsed * 1.0f / 500;
		px_float s = 1 + a * 2;
		px_color clr = pdesc->type == PX_OBJECT_ROBOT_TYPE_RED ? PX_COLOR_RED : PX_COLOR_BLUE;
		clr._argb.a = (px_byte)(255 * (1 - a));
		PX_GeoDrawBorder(prendersurface, (px_int)(x - (PX_OBJECT_CELL_PIXELS / 2 * s)),
			(px_int)(y - (PX_OBJECT_CELL_PIXELS / 2 * s)),
			(px_int)(x + (PX_OBJECT_CELL_PIXELS / 2 * s)),
			(px_int)(y + (PX_OBJECT_CELL_PIXELS / 2 * s)),
			(px_int)(s * 2),
			clr
		);

	}

	//texture
	do 
	{
		px_texture statictex = {0};
		statictex.height = 48;
		statictex.limit_bottom = 47;
		statictex.limit_right = 47;
		statictex.MP = 0;
		statictex.surfaceBuffer = (px_color*)pdesc->data.profile_data;
		statictex.width = 48;

		PX_TextureRender(prendersurface, &statictex, (px_int)pObject->x, (px_int)pObject->y, PX_ALIGN_CENTER, 0);

		if (pdesc->vm.pThread[0].suspend||pdesc->vm.Suspend)
		{
			PX_GeoDrawRect(prendersurface, 0, 0, 47, 47, PX_COLOR(64, 255, 0, 0));
		}

	} while (0);

	//draw breath
	pdesc->breath_elapsed += elapsed;
	if (pdesc->breath_elapsed >= 2000)
		pdesc->breath_elapsed -= 2000;
	do
	{
		px_color clr;
		px_float a = PX_ABS(1000 - (px_int)pdesc->breath_elapsed) / 1000.f;
		if (pdesc->type == PX_OBJECT_ROBOT_TYPE_RED)
		{
			clr = PX_COLOR((px_byte)(a * 255), 255, 0, 0);
		}
		else
		{
			clr = PX_COLOR((px_byte)(a * 255), 0, 0, 255);
		}

		PX_GeoDrawBorder(prendersurface, (px_int)(pObject->x- pObject->Width/2), (px_int)(pObject->y - pObject->Height / 2), (px_int)(pObject->x + pObject->Width/2), (px_int)(pObject->y + pObject->Height/2), 3, clr);
	} while (0);
	
	//name
	do 
	{
		if (pdesc->type==PX_OBJECT_ROBOT_TYPE_RED)
		{
			PX_FontModuleDrawText(prendersurface, pdesc->fm, (px_int)pObject->x, (px_int)(pObject->y - pObject->Height / 2 - 2), PX_ALIGN_MIDBOTTOM, pdesc->data.name, PX_COLOR_RED);
		}
		if (pdesc->type == PX_OBJECT_ROBOT_TYPE_BLUE)
		{
			PX_FontModuleDrawText(prendersurface, pdesc->fm, (px_int)pObject->x, (px_int)(pObject->y-pObject->Height/2-2), PX_ALIGN_MIDBOTTOM, pdesc->data.name, PX_COLOR_BLUE);
		}
		
	} while (0);

}

const px_char *PX_Object_RobotGetStdlib()
{
	return stdlib;
}

px_void PX_Object_RobotOnImpact(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_float x, y, w, h;
	PX_Object_Robot* pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	PX_OBJECT_INHERIT_CODE(pObject,x, y, w, h);
	if (!pdesc->death)
	{
		pdesc->death = PX_TRUE;
		PX_WorldAddObject(pdesc->pworld,PX_Object_GameExplosionCreate(pdesc->mp, pdesc->pworld, x, y, 32, 1, 5, 3000,(px_dword)pdesc->type));
		if (pdesc->pruntime)
			PX_SoundPlayAdd(&pdesc->pruntime->soundplay, PX_SoundCreate(PX_ResourceLibraryGetSound(&pdesc->pruntime->ResourceLibrary, "explosion"), 0));
	}
	
}


PX_Object* PX_Object_RobotCreate(px_memorypool* mp, PX_Runtime* pruntime, PX_Object* parent,
	px_float x, px_float y,
	PX_World* pworld,
	PX_OBJECT_ROBOT_TYPE type,
	PX_GameInstanceData* pdata,
	PX_Object* map,
	PX_FontModule *fm)
{
	PX_Object* pObject;
	PX_Object_Robot desc,*pdesc;
	PX_memset(&desc, 0, sizeof(desc));
	desc.fm = fm;
	desc.mp = mp;
	desc.map = map;
	desc.data = *pdata;
	desc.pworld = pworld;
	desc.type = type;
	desc.velocity = PX_POINT2D(0, 0);
	desc.pruntime = pruntime;
	pObject = PX_ObjectCreateEx(mp,parent,x,y, PX_OBJECT_ROBOT_Z, PX_OBJECT_CELL_PIXELS-0.1f, PX_OBJECT_CELL_PIXELS-0.1f,0,0, PX_Object_RobotUpdate, PX_Object_RobotRender,0,&desc,sizeof(desc));
	pdesc = PX_ObjectGetDesc(PX_Object_Robot, pObject);
	if (!PX_VMInitialize(&pdesc->vm, mp, pdata->bin, pdata->binsize))return 0;
	if (pdesc->vm.VM_Globalsize > PX_OBJECT_ROBOT_GLOBAL_SIZE)
	{
		PX_VMFree(&pdesc->vm);
		return 0;
	}
	if (pdesc->vm.VM_Stacksize > PX_OBJECT_ROBOT_STACK_SIZE)
	{
		PX_VMFree(&pdesc->vm);
		return 0;
	}
	PX_VMRegistHostFunction(&pdesc->vm, "MoveLeft", PX_Object_Robot_VmMoveLeft, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "MoveRight", PX_Object_Robot_VmMoveRight, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "MoveUp", PX_Object_Robot_VmMoveUp, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "MoveDown", PX_Object_Robot_VmMoveDown, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "EnemyX", PX_Object_Robot_VmGetEnemyPosX, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "EnemyY", PX_Object_Robot_VmGetEnemyPosY, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "MyX", PX_Object_Robot_VmGetMyPosX, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "MyY", PX_Object_Robot_VmGetMyPosY, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "Map", PX_Object_Robot_VmGetMap, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "Fire", PX_Object_Robot_VmFire, pObject);
	PX_VMRegistHostFunction(&pdesc->vm, "Rand", PX_Object_Robot_VmRand, pObject);
	PX_VMRunFunction(&pdesc->vm, 0, "_BOOT", 0, 0);
	PX_VMBeginThreadFunction(&pdesc->vm, 0, "main", 0, 0);
	if (type==PX_OBJECT_ROBOT_TYPE_RED)
	{
		pObject->impact_object_type = 1;
		pObject->impact_target_type = 2;
		pdesc->direction = PX_POINT2D(1, 0);
	}
	else
	{
		pObject->impact_object_type = 2;
		pObject->impact_target_type = 1;
		pdesc->direction = PX_POINT2D(-1, 0);
	}
	

	pdesc->vm_seed = pdata->seed;
	PX_ObjectRegisterEvent(pObject, PX_OBJECT_EVENT_IMPACT, PX_Object_RobotOnImpact, pObject);
	return pObject;
}

