#include "PainterEngine_Application.h"
PX_Application App;

px_void PX_Apllication_OnUpToDateDone(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_Object_RankRefresh(pApp->page_rank);
}

px_void PX_Apllication_OnRankRefresh(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_Object_ClientUpToDateRefresh(pApp->function_uptodate);
}

px_void PX_Apllication_OnRankJoinEnter(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_Object_BattleRefresh(pApp->page_battle);
	PX_Object_BattleResetData(pApp->page_battle);
	pApp->page_battle->Visible = PX_TRUE;
	pApp->page_join->Visible = PX_FALSE;
	pApp->page_rank->Visible = PX_FALSE;
	pApp->page_server->Visible = PX_FALSE;
}

px_void PX_Application_OnSwitchMatch(px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	pApp->page_battle->Visible = PX_FALSE;
	pApp->page_join->Visible = PX_FALSE;
	pApp->page_rank->Visible = PX_FALSE;
	pApp->page_match->Visible = PX_TRUE;
	pApp->page_server->Visible = PX_FALSE;
}


px_void PX_Application_OnSwitchRank(px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_Apllication_OnRankRefresh(0, PX_OBJECT_BUILD_EVENT(PX_OBJECT_RANK_EVENT_REFRESH), ptr);
	pApp->page_battle->Visible = PX_FALSE;
	pApp->page_join->Visible = PX_FALSE;
	pApp->page_rank->Visible = PX_TRUE;
	pApp->page_match->Visible = PX_FALSE;
	pApp->page_server->Visible = PX_FALSE;
}

px_void PX_Application_OnSwitchJoin(px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	pApp->page_battle->Visible = PX_FALSE;
	pApp->page_join->Visible = PX_TRUE;
	pApp->page_rank->Visible = PX_FALSE;
	pApp->page_match->Visible = PX_FALSE;
	pApp->page_server->Visible = PX_FALSE;
	PX_Object_BattleRefresh(pApp->page_battle);
}


px_void PX_Application_OnSwitchBattle(px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	pApp->page_battle->Visible = PX_TRUE;
	pApp->page_join->Visible = PX_FALSE;
	pApp->page_rank->Visible = PX_FALSE;
	pApp->page_server->Visible = PX_FALSE;
	pApp->page_match->Visible = PX_FALSE;
	PX_Object_BattleRefresh(pApp->page_battle);
}

px_void PX_Application_OnSwitchServer(px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	pApp->page_battle->Visible = PX_FALSE;
	pApp->page_join->Visible = PX_FALSE;
	pApp->page_rank->Visible = PX_FALSE;
	pApp->page_match->Visible = PX_FALSE;
	pApp->page_server->Visible = PX_TRUE;
}





px_bool PX_ApplicationInitialize(PX_Application *pApp,px_int screen_width,px_int screen_height)
{
	PX_ApplicationInitializeDefault(&pApp->runtime, screen_width, screen_height);
	PX_srand(PX_TimeGetSecond() + PX_TimeGetHour() * 100 + PX_TimeGetDay()*10000);
	if (!PX_JsonInitialize(&pApp->runtime.mp_resources, &pApp->Language))return PX_FALSE;
	if (!PX_LoadJsonFromFile(&pApp->Language, "assets/language.json")) {
		printf("assets/language.json error\n");
		return PX_FALSE;
	}


	if (!PX_JsonInitialize(&pApp->runtime.mp_resources, &pApp->config))return PX_FALSE;
	if (!PX_LoadJsonFromFile(&pApp->config, "assets/config.json")) {
		printf("assets/config.json error\n");
		return PX_FALSE;
	}

	if (!PX_FontModuleInitialize(&pApp->runtime.mp_resources, &pApp->runtime.fontmodule))return PX_FALSE;
	if (!PX_LoadFontModuleFromFile(&pApp->runtime.fontmodule, "assets/gbk_18.pxf"))
	{
		printf("assets/gbk_18.pxf missing\n");
		return PX_FALSE;
	}

	if (!PX_LoadSoundToResource(&pApp->runtime.ResourceLibrary, "assets/fire.wav", "fire"))
	{
		printf("fire.wav missing\n");
		return PX_FALSE;
	}
	if (!PX_LoadSoundToResource(&pApp->runtime.ResourceLibrary, "assets/explosion.wav", "explosion")) {
		printf("explosion.wav missing\n");
		return PX_FALSE;
	}

	if (!PX_SoundPlayInitialize(&pApp->runtime.mp_ui, &pApp->runtime.soundplay))return PX_FALSE;

	if (PX_JsonGetBoolean(&pApp->config, "server.enable") == PX_FALSE)
	{
		if (!PX_AudioInitialize(&pApp->runtime.soundplay)) return PX_FALSE;
	}
	


	pApp->root = PX_ObjectCreate(&pApp->runtime.mp_ui, 0, 0, 0, 0, 0, 0, 0);
	pApp->menu = PX_Object_MenuCreate(&pApp->runtime.mp_ui, pApp->root, 0, 0, 64, &pApp->runtime.fontmodule);
	pApp->servermp = MP_Create(MP_Malloc(&pApp->runtime.mp, PX_APPLICATION_MEMORYPOOL_SERVER_SIZE), PX_APPLICATION_MEMORYPOOL_SERVER_SIZE);
	
	PX_Object_MenuAddItem(pApp->menu, 0, PX_JsonGetString(&pApp->Language, "menu.match"), PX_Application_OnSwitchMatch, pApp);
	PX_Object_MenuAddItem(pApp->menu, 0, PX_JsonGetString(&pApp->Language, "menu.rank"), PX_Application_OnSwitchRank, pApp);
	PX_Object_MenuAddItem(pApp->menu, 0, PX_JsonGetString(&pApp->Language, "menu.join"), PX_Application_OnSwitchJoin, pApp);
	PX_Object_MenuAddItem(pApp->menu, 0, PX_JsonGetString(&pApp->Language, "menu.battle"), PX_Application_OnSwitchBattle, pApp);
	PX_Object_MenuAddItem(pApp->menu, 0, PX_JsonGetString(&pApp->Language, "menu.server"), PX_Application_OnSwitchServer, pApp);

	pApp->page_match = PX_Object_MatchCreate(&pApp->runtime.mp_ui, pApp->root, 0, 24, pApp->runtime.surface_width * 1.0f, pApp->runtime.surface_height * 1.0f - 24, &pApp->Language, &pApp->config, &pApp->runtime.fontmodule);
	if (!pApp->page_match)return 0;

	pApp->page_rank = PX_Object_RankCreate(&pApp->runtime.mp_ui, pApp->root, 0, 24, pApp->runtime.surface_width*1.0f, pApp->runtime.surface_height*1.0f - 24, &pApp->Language, &pApp->config, &pApp->runtime.fontmodule);
	if (!pApp->page_rank)return 0;
	pApp->page_rank->Visible = PX_FALSE;

	pApp->page_join = PX_Object_JoinCreate(&pApp->runtime.mp_ui, pApp->root, 0,0, &pApp->Language, &pApp->config, &pApp->runtime.fontmodule);
	if (!pApp->page_join)
	{
		return 0;
	}
	pApp->page_join->x = pApp->runtime.surface_width / 2.f-220;
	pApp->page_join->y = pApp->runtime.surface_height / 2.f-240;
	pApp->page_join->Visible = PX_FALSE;
	

	pApp->page_battle = PX_Object_BattleCreate(&pApp->runtime, pApp->root, &pApp->Language, &pApp->config, &pApp->runtime.fontmodule);
	if (!pApp->page_battle) {
		printf("battle failed"); return 0;
	}
	pApp->page_battle->Visible = PX_FALSE;
	
	pApp->function_uptodate = PX_Object_ClientUpToDateCreate(&pApp->runtime.mp, pApp->root, &pApp->Language, &pApp->config, &pApp->runtime.fontmodule);
	if (!pApp->function_uptodate) {
		printf("up-to-date failed"); return 0;
	}

	pApp->page_server = PX_Object_ServerCreate(&pApp->servermp, pApp->root, 0, 24, pApp->runtime.surface_width * 1.0f, pApp->runtime.surface_height - 24.f, &pApp->config, &pApp->runtime.fontmodule);
	if (!pApp->page_server)
	{
		printf("Server failed");
		return 0;
	}
	pApp->page_server->Visible = PX_FALSE;

	PX_ObjectRegisterEvent(pApp->root, PX_OBJECT_CLIENTUPTODATE_DONE, PX_Apllication_OnUpToDateDone,pApp);
	PX_ObjectRegisterEvent(pApp->root, PX_OBJECT_RANK_EVENT_REFRESH, PX_Apllication_OnRankRefresh, pApp);
	PX_ObjectRegisterEvent(pApp->root, PX_OBJECT_JOIN_EVENT_TEST, PX_Apllication_OnRankJoinEnter, pApp);
	PX_ObjectRegisterEvent(pApp->root, PX_OBJECT_RANK_EVENT_JOIN, PX_Apllication_OnRankJoinEnter, pApp);
	return PX_TRUE;
}

px_void PX_ApplicationUpdate(PX_Application *pApp,px_dword elapsed)
{
	PX_ObjectUpdate(pApp->root, elapsed);
}

px_void PX_ApplicationRender(PX_Application *pApp,px_dword elapsed)
{
	px_surface *pRenderSurface=&pApp->runtime.RenderSurface;
	PX_RuntimeRenderClear(&pApp->runtime,PX_OBJECT_UI_DEFAULT_BACKGROUNDCOLOR);
	PX_ObjectRender(pRenderSurface, pApp->root, elapsed);
}

px_void PX_ApplicationPostEvent(PX_Application *pApp,PX_Object_Event e)
{
	PX_ApplicationEventDefault(&pApp->runtime, e);
	PX_ObjectPostEvent(pApp->root, e);
}

